//Moveable and Resizeable TGraphicControl at runtime
//Created by Tomas Koutny.
//Version 1.0

//------------------------------------------------------------------------------
//WARRANTIES AND DISCLAIMER:
//
//EXCEPT AS EXPRESSLY PROVIDED OTHERWISE IN A WRITTEN AGREEMENT BETWEEN
//Tomas Koutny ("LICENSOR") AND YOU, THE LICENSED WORKS ARE NOW PROVIDED "AS IS"
//WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT
//LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//FOR A PARTICULAR PURPOSE, OR THE WARRANTY OF NON-INFRINGEMENT.

//IN NO EVENT SHALL LICENSOR OR ITS SUPPLIERS BE LIABLE TO YOU OR ANY THIRD
//PARTY FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
//KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING, WITHOUT LIMITATION, THOSE
//RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT LICENSOR HAD BEEN
//ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, AND ON ANY THEORY OF LIABILITY,
//ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE LICENSED WORKS. SOME
//JURISDICTIONS PROHIBIT THE EXCLUSION OR LIMITATION OF LIABILITY FOR
//CONSEQUENTIAL OR INCIDENTAL DAMAGES, SO THE ABOVE LIMITATIONS MAY NOT APPLY TO
//YOU. THESE LIMITATIONS SHALL APPLY NOTWITHSTANDING ANY FAILURE OF ESSENTIAL
//PURPOSE OF ANY LIMITED REMEDY.
//------------------------------------------------------------------------------

//See www.rawos.com for more software and resources.


unit FloatBar;

interface

uses
  Windows, Classes, Graphics, Controls, menus;

const
  fbaNone = 0;       //nothing happening
  fbaSizing = 1;     //the size of control is changing
  fbaMoving = 2;     //the control is moving

type
  TFloatBar = class(TGraphicControl)
  private
    FBorder:TColor;                 //color of our background
    FBorderWidth:integer;
    FOldTop,
    FOldLeft,
    FOldWidth,
    FOldHeight:integer;             //for AfterMove needs
    FPopupMenu:TPopupMenu;          //let's be user-friendly
  protected
    FAction:integer;                //current action, fbaXXXX
    SizeKind:integer;               //how to change size in case of resize
                                    //look at result AtBorderArea
    OldX, OldY:integer;             //help coordinates for resize and move
    NoPaint:boolean;                //when to disable repainting
    procedure FSetBackground(col:TColor);
    procedure DoClick(Sender:TObject);
    procedure AfterMove(from:TRect); virtual;   
				     //called after move from location from						
				     //from stores old position and size	
    function AtBorderArea(x, y:integer):integer; virtual;
			   //determinis position of mouse in area of control
                           //returns : 0 - not at border
                                     //1 - left border
                                     //2 - left upper corner
                                     //3 - upper border
                                     //4 - right upper corner
                                     //5 - right border
                                     //6 - right down corner
                                     //7 - down border
                                     //8 - left down corner
    procedure ShowPopupMenu(x,y:integer); virtual;
              //raises popup menu if assigned
  public
    procedure Paint; override;  
    constructor Create(AOwner:TComponent);  override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override; 
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
                                override; //begin of resize or move
                                    //change cursor
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
                                override; //end of resize or move
    function CanMoveAt(var x, y:integer):boolean; virtual; 
				//oppurtunity for descendant to suppress
				//move to specific location
    function CanSizeAt(AWidth, AHeight:integer):boolean; virtual;
				//oppurtunity for descendant to suppress
				//resize to specific size
    procedure MoveAt(ALeft, ATop:integer); virtual; //moves self at [ALeft, ATop]
  published
    property Border:TColor read FBorder write FBorder;
    property BorderWidth:integer read FBorderWidth write FBorderWidth;
    property Canvas;
    property OnClick;
    property PopupMenu:TPopupMenu read FPopupMenu write FPopupMenu;
  end;

procedure Register;

implementation

constructor TFloatBar.Create(AOwner:TComponent);
begin
  inherited;

  ControlStyle := ControlStyle + [csReplicatable, csDisplayDragImage];

  Parent:=(AOwner as TWinControl);

  FBorderWidth:=5;
  Width:=70;
  Height:=30;
end;

procedure TFloatBar.Paint;
begin
  canvas.brush.color:=FBorder;
  canvas.Framerect(rect(0, 0, width, height));
end;

procedure TFloatBar.FSetBackground(col:TColor);
begin
  FBorder:=col;
  Paint;
end;

function TFloatBar.AtBorderArea(x, y:integer):integer;
var l, r, t, b: boolean;
begin
  //vraci : 0 - neni na okraji
  //        1 - levy okraj
  //        2 - levy horni roh
  //        3 - horni okraj
  //        4 - pravy horni roh
  //        5 - pravy okraj
  //        6 - pravy dolni roh
  //        7 - dolni okraj
  //        8 - levy dolni roh

  //zjistime, jak jsme na tom s pozici kurzoru
  l:=x < FBorderWidth;         //levy okraj
  t:=y < FBorderWidth;         //horni okraj
  r:=x > Width - FBorderWidth; //pravy okraj
  b:=y > Height - FBorderWidth;//dolni okraj

  result:=0; //neco predpokladat musime
  if l then
     begin //kruzor je v leve casti
        result:=1;
        if t then result:=2; //levy horni roh
        if b then result:=8; //levy dolni roh
     end;

  if r then
     begin //kruzor je v prave casti
        result:=5;
        if t then result:=4; //pravy horni roh
        if b then result:=6; //pravy dolni roh
     end;

  if not(l or r) then
     begin //neni ani levy ani horni okraj
       if t then result:=3;  //horni okraj
       if b then result:=7;  //dolni okraj
     end;
end;

procedure TFloatBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var incx, incy:integer;

procedure AdjustX;     //viz pouziti
var New:integer;
begin
  if (SizeKind>2) and (SizeKind<8) then
     begin                             //pohyb doprava
       New:=width+incx;
       if not CanSizeAt(New, Height) then exit;
       width:=New;
       OldX:=X;
     end else
         begin
           New:=width-incx;
           if CanSizeAt(New, Height) then
              begin
                Left:=left+incx;
                Width:=New;
              end;
         end;
end;

procedure AdjustY;     //viz pouziti
var New:Integer;
begin
  if SizeKind>4 then   //pohyb dolu
     begin
       New:=height+incy;
       if not CanSizeAt(Width, New) then exit;
       Height:=New;
       OldY:=Y;
     end else
         begin
           New:=Height-incy;
           if CanSizeAt(Width, New) then
              begin
                top:=top+incy;
                Height:=New;
              end;
         end;
end;

begin
  if FAction = fbaNone then //neprobiha zadna akce => zkusime nabudnout resize
     case AtBorderArea(x, y) of
        1:Cursor:=crSizeWE;// levy okraj
        2:Cursor:=crSizeNWSE;// levy horni roh
        3:Cursor:=crSizeNS;// horni okraj
        4:Cursor:=crSizeNESW;// pravy horni roh
        5:Cursor:=crSizeWE;// pravy okraj
        6:Cursor:=crSizeNWSE;// pravy dolni roh
        7:Cursor:=crSizeNS;// dolni okraj
        8:Cursor:=crSizeNESW;// levy dolni roh
      else Cursor:=crDefault;//0 nebo jina moznost- neni na okraji
    end;

  if FAction = fbaSizing then  //probiha zmena velikosti
     begin
       //nejrpve stanovime jednotlive inkrementy
       incy:=y-OldY;
       incx:=x-OldX;

       //a podle nich upravime velikost
       case SizeKind of
          1, 5:AdjustX;  // levy a pravy okraj
          2, 4, 6, 8:
            begin     // rohy
              AdjustX;
              AdjustY;
            end;
          3, 7:AdjustY; // horni a dolni okraj
       end;
     end;

  if (FAction = fbaMoving) then
     begin
       //predvypocitame hodnoty, kam by jsme se pohli
       incx:=left+x-Oldx;
       incy:=top+y-OldY;
       if CanMoveAt(incx, incy) then //zeptame se, jestli to jde
          begin
            top:=incy;  //zmena na ose x
            left:=incx;//a y
          end;
     end;
end;

procedure TFloatBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  
  if ssLeft in Shift then
     begin
       SizeKind:=AtBorderArea(x, y);//pokud budeme menit velikost, tak jak
       OldX:=x;           //souradnice, na kterych se to stalo
       OldY:=y;

       FOldTop:=Top;
       FOldLeft:=Left;
       FOldWidth:=Width;
       FOldHeight:=Height;

      if SizeKind <> 0 then FAction:=fbaSizing //budeme menit velikost
         else begin            //budeme menit pozici
               cursor:=crSize;
               FAction:=fbaMoving;
             end;
     end else
         if (ssRight in Shift) then ShowPopupMenu(x,y);
end;

procedure TFloatBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  cursor:=crDefault;      //vratime vse do normalu
  FAction:=fbaNone;
  AfterMove(rect(FOldLeft, FOldTop, FOldLeft+FOldWidth, FOldTop+FOldHeight));
end;

function TFloatBar.CanMoveAt(var x, y:integer):boolean;
begin
  result:=true;
end;

function TFloatBar.CanSizeAt(AWidth, AHeight:integer):boolean;
begin
  result:=(AWidth > 0) and (AHeight > 0);
end;

procedure TFloatBar.AfterMove(from:TRect);
begin
end;

procedure TFloatBar.DoClick(Sender:TObject);
begin
  if Assigned(OnClick) then OnClick(Self);
end;

procedure TFloatBar.MoveAt(ALeft, ATop:integer);
begin
  FOldLeft:=Left;
  FOldTop:=Top;

  Left:=ALeft;
  Top:=ATop;
  AfterMove(rect(FOldLeft, FOldTop, FOldLeft+Width, FOldTop+Height));
end;

procedure TFloatBar.ShowPopupMenu(x,y:integer);
var tp:TPoint;
begin
  if not assigned(FPopupMenu) then exit;

  tp:=ClientToScreen(Point(x, y));
  FPopupMenu.Popup(tp.x, tp.y);
end;

procedure Register;
begin
  RegisterComponents('XDTK', [TFloatBar]);
end;

end.
